/*
 * uart_bsp.h
 *
 *  Created on: Jul 11, 2023
 *      Author: hussamaldean
 */

#ifndef UART_BSP_H_
#define UART_BSP_H_

#include "stdint.h"

#include "stm32f4xx.h"


typedef enum
{
	UART_SUCCESS=0,
	UART_FAILED =-1,
	UART_TIMEOUT=-2
}UART_StatusTypedef;

/**
 * @brief This enum represents the direction of UART
 */

typedef enum
{

	UART_TX_Only=0,
	UART_RX_Only=1,
	UART_TX_RX=2

}UART_DirectionTypedef;


typedef enum
{
	Interrupt_disabled=0, /*No interrupt*/
	Interrupt_Enabled=1, /*Interrupt enabled*/

	TXEIE_Enable=1, /*Transmit buffer is empty*/
	RXNEIE_Enable=1, /*Receiver buffer is not empty*/
	IDLEIE_Enable=1, /*IDLE Line detection enabled*/

}Interrupt_Source_Typedef;

typedef enum
{
	usart1=0,
	usart2=1,
	uart6=2,
}USART_NumberTypedef;


typedef struct
{
	uint32_t busSpeed;
	uint32_t buadRate;
	uint8_t mode;
	uint8_t InterrupteEnbable;
	uint8_t TX_InterruptEnable;
	uint8_t RX_InterruptEnable;
	uint8_t IDLE_Line_Interrupt_Enable;
	uint8_t uart_number;
}UART_ConfigTypedef;


void UART2_CLOCK_ENABLE(void);

void BSP_UART_Init(USART_TypeDef *uart, UART_ConfigTypedef *config);

UART_StatusTypedef BSP_UART_Write_Char(USART_TypeDef *uart,char ch, uint32_t timeout);

UART_StatusTypedef BSP_UART_Read_Char(USART_TypeDef *uart,char *recChar, uint32_t timeout);


#endif /* UART_BSP_H_ */
