/*
 * dma_bsp.c
 *
 *  Created on: Aug 9, 2023
 *      Author: hussamaldean
 */


#include "dma_bsp.h"

void BSP_DMA_Init(DMA_Stream_TypeDef *dma, DMA_ConfigTypedef *config )
{
	/*Disable the stream*/
	dma->CR &=~ DMA_SxCR_EN;
	/*Wait until stream is disabled*/
	while ((dma->CR& DMA_SxCR_EN));

	dma->CR=0;

	dma->CR |=(config->DMA_Channel << DMA_SxCR_CHSEL_Pos);

	dma->CR |=(config->PeripheralBurstMode<<DMA_SxCR_PBURST_Pos);

	dma->CR|=(config->MemoryBurstMode<<DMA_SxCR_MBURST_Pos);

	dma->CR|=(config->DoubleBuffer<<DMA_SxCR_DBM_Pos);

	dma->CR|=(config->Priority<<DMA_SxCR_PL_Pos);

	dma->CR|=(config->MemorySize<<DMA_SxCR_MSIZE_Pos);

	dma->CR|=(config->PerpheralSize<<DMA_SxCR_PSIZE_Pos);

	dma->CR|= (config->MemoryIncMode<<DMA_SxCR_MINC_Pos);

	dma->CR|= (config->PeripheralIncMode<<DMA_SxCR_PINC_Pos);

	dma->CR|=(config->CircularMode<<DMA_SxCR_CIRC_Pos);

	dma->CR|=(config->Direction<<DMA_SxCR_DIR_Pos);

	if(config->Direction==memory2Memory)
	{
		dma->FCR|=(1U<<DMA_SxFCR_DMDIS_Pos);
	}

	dma->CR|=(config->PeripheralController<<DMA_SxCR_PFCTRL_Pos);

	if(config->InterruptEnable==InterruptEnbaled)
	{

		switch (config->DMAStream)
		{

		case dma1stream0 : NVIC_EnableIRQ(DMA1_Stream0_IRQn); break;
		case dma1stream1 : NVIC_EnableIRQ(DMA1_Stream1_IRQn); break;
		case dma1stream2 : NVIC_EnableIRQ(DMA1_Stream2_IRQn); break;
		case dma1stream3 : NVIC_EnableIRQ(DMA1_Stream3_IRQn); break;
		case dma1stream4 : NVIC_EnableIRQ(DMA1_Stream4_IRQn); break;
		case dma1stream5 : NVIC_EnableIRQ(DMA1_Stream5_IRQn); break;
		case dma1stream6 : NVIC_EnableIRQ(DMA1_Stream6_IRQn); break;
		case dma1stream7 : NVIC_EnableIRQ(DMA1_Stream7_IRQn); break;

		case dma2stream0 : NVIC_EnableIRQ(DMA2_Stream0_IRQn); break;
		case dma2stream1 : NVIC_EnableIRQ(DMA2_Stream1_IRQn); break;
		case dma2stream2 : NVIC_EnableIRQ(DMA2_Stream2_IRQn); break;
		case dma2stream3 : NVIC_EnableIRQ(DMA2_Stream3_IRQn); break;
		case dma2stream4 : NVIC_EnableIRQ(DMA2_Stream4_IRQn); break;
		case dma2stream5 : NVIC_EnableIRQ(DMA2_Stream5_IRQn); break;
		case dma2stream6 : NVIC_EnableIRQ(DMA2_Stream6_IRQn); break;
		case dma2stream7 : NVIC_EnableIRQ(DMA2_Stream7_IRQn); break;

		default: break;


		}

	}

	dma->CR|=(config->TransferCompleteInterrupt<<DMA_SxCR_TCIE_Pos);

	dma->CR|=(config->HalfTransferInterrupt<<DMA_SxCR_HTIE_Pos);

	dma->CR|=(config->TransferErrorInterrupt<<DMA_SxCR_TEIE_Pos);

	dma->CR|=(config->DirectModeErrorInterrupt<<DMA_SxCR_DMEIE_Pos);


}

void BSP_DMA_Transfer_Single_Buffer(DMA_Stream_TypeDef *dma, uint32_t src, uint32_t dest, uint16_t len)
{
	dma->M0AR=dest;
	dma->PAR=src;
	dma->NDTR=len;
	dma->CR |=DMA_SxCR_EN;
}

void BSP_DMA_Transfer_Double_Buffer(DMA_Stream_TypeDef *dma, uint32_t src, uint32_t dest1,uint32_t dest2, uint16_t len)
{
	dma->M0AR=(uint32_t)dest1;
	dma->M1AR=(uint32_t)dest2;
	dma->PAR=(uint32_t)src;
	dma->NDTR=len;
	dma->CR |=DMA_SxCR_EN;
}


__WEAK void DMA1_Stream0_handler(void)
{}
__WEAK void DMA1_Stream1_handler(void)
{}
__WEAK void DMA1_Stream2_handler(void)
{}
__WEAK void DMA1_Stream3_handler(void)
{}
__WEAK void DMA1_Stream4_handler(void)
{}
__WEAK void DMA1_Stream5_handler(void)
{}
__WEAK void DMA1_Stream6_handler(void)
{}
__WEAK void DMA1_Stream7_handler(void)
{}

__WEAK void DMA2_Stream0_handler(void)
{}
__WEAK void DMA2_Stream1_handler(void)
{}
__WEAK void DMA2_Stream2_handler(void)
{}
__WEAK void DMA2_Stream3_handler(void)
{}
__WEAK void DMA2_Stream4_handler(void)
{}
__WEAK void DMA2_Stream5_handler(void)
{}
__WEAK void DMA2_Stream6_handler(void)
{}
__WEAK void DMA2_Stream7_handler(void)
{}


void DMA1_Stream0_IRQHandler(void)
{
	/*Call DMA handler*/
	DMA1_Stream0_handler();
	/*Clear interrupt flags*/
	DMA1->LIFCR=DMA_LIFCR_CFEIF0;
	DMA1->LIFCR=DMA_LIFCR_CDMEIF0;
	DMA1->LIFCR=DMA_LIFCR_CTEIF0;
	DMA1->LIFCR=DMA_LIFCR_CHTIF0;
	DMA1->LIFCR=DMA_LIFCR_CTCIF0;



}

void DMA1_Stream1_IRQHandler(void)
{
	DMA1_Stream1_handler();

	DMA1->LIFCR=DMA_LIFCR_CFEIF1;
	DMA1->LIFCR=DMA_LIFCR_CDMEIF1;
	DMA1->LIFCR=DMA_LIFCR_CTEIF1;
	DMA1->LIFCR=DMA_LIFCR_CHTIF1;
	DMA1->LIFCR=DMA_LIFCR_CTCIF1;
}
void DMA1_Stream2_IRQHandler(void)
{
	DMA1_Stream2_handler();

	DMA1->LIFCR=DMA_LIFCR_CFEIF2;
	DMA1->LIFCR=DMA_LIFCR_CDMEIF2;
	DMA1->LIFCR=DMA_LIFCR_CTEIF2;
	DMA1->LIFCR=DMA_LIFCR_CHTIF2;
	DMA1->LIFCR=DMA_LIFCR_CTCIF2;
}
void DMA1_Stream3_IRQHandler(void)
{
	DMA1_Stream3_handler();

	DMA1->LIFCR=DMA_LIFCR_CFEIF3;
	DMA1->LIFCR=DMA_LIFCR_CDMEIF3;
	DMA1->LIFCR=DMA_LIFCR_CTEIF3;
	DMA1->LIFCR=DMA_LIFCR_CHTIF3;
	DMA1->LIFCR=DMA_LIFCR_CTCIF3;
}
void DMA1_Stream4_IRQHandler(void)
{
	DMA1_Stream4_handler();

	DMA1->HIFCR=DMA_HIFCR_CFEIF4;
	DMA1->HIFCR=DMA_HIFCR_CDMEIF4;
	DMA1->HIFCR=DMA_HIFCR_CTEIF4;
	DMA1->HIFCR=DMA_HIFCR_CHTIF4;
	DMA1->HIFCR=DMA_HIFCR_CTCIF4;
}
void DMA1_Stream5_IRQHandler(void)
{
	DMA1_Stream5_handler();

	DMA1->HIFCR=DMA_HIFCR_CFEIF5;
	DMA1->HIFCR=DMA_HIFCR_CDMEIF5;
	DMA1->HIFCR=DMA_HIFCR_CTEIF5;
	DMA1->HIFCR=DMA_HIFCR_CHTIF5;
	DMA1->HIFCR=DMA_HIFCR_CTCIF5;
}

void DMA1_Stream6_IRQHandler(void)
{
	DMA1_Stream6_handler();

	DMA1->HIFCR=DMA_HIFCR_CFEIF6;
	DMA1->HIFCR=DMA_HIFCR_CDMEIF6;
	DMA1->HIFCR=DMA_HIFCR_CTEIF6;
	DMA1->HIFCR=DMA_HIFCR_CHTIF6;
	DMA1->HIFCR=DMA_HIFCR_CTCIF6;
}
void DMA1_Stream7_IRQHandler(void)
{
	DMA1_Stream7_handler();

	DMA1->HIFCR=DMA_HIFCR_CFEIF7;
	DMA1->HIFCR=DMA_HIFCR_CDMEIF7;
	DMA1->HIFCR=DMA_HIFCR_CTEIF7;
	DMA1->HIFCR=DMA_HIFCR_CHTIF7;
	DMA1->HIFCR=DMA_HIFCR_CTCIF7;
}


void DMA2_Stream0_IRQHandler(void)
{
	DMA2_Stream0_handler();

	DMA2->LIFCR=DMA_LIFCR_CFEIF0;
	DMA2->LIFCR=DMA_LIFCR_CDMEIF0;
	DMA2->LIFCR=DMA_LIFCR_CTEIF0;
	DMA2->LIFCR=DMA_LIFCR_CHTIF0;
	DMA2->LIFCR=DMA_LIFCR_CTCIF0;
}

void DMA2_Stream1_IRQHandler(void)
{
	DMA2_Stream1_handler();

	DMA2->LIFCR=DMA_LIFCR_CFEIF1;
	DMA2->LIFCR=DMA_LIFCR_CDMEIF1;
	DMA2->LIFCR=DMA_LIFCR_CTEIF1;
	DMA2->LIFCR=DMA_LIFCR_CHTIF1;
	DMA2->LIFCR=DMA_LIFCR_CTCIF1;
}
void DMA2_Stream2_IRQHandler(void)
{
	DMA2_Stream2_handler();

	DMA2->LIFCR=DMA_LIFCR_CFEIF2;
	DMA2->LIFCR=DMA_LIFCR_CDMEIF2;
	DMA2->LIFCR=DMA_LIFCR_CTEIF2;
	DMA2->LIFCR=DMA_LIFCR_CHTIF2;
	DMA2->LIFCR=DMA_LIFCR_CTCIF2;
}
void DMA2_Stream3_IRQHandler(void)
{
	DMA2_Stream3_handler();

	DMA2->LIFCR=DMA_LIFCR_CFEIF3;
	DMA2->LIFCR=DMA_LIFCR_CDMEIF3;
	DMA2->LIFCR=DMA_LIFCR_CTEIF3;
	DMA2->LIFCR=DMA_LIFCR_CHTIF3;
	DMA2->LIFCR=DMA_LIFCR_CTCIF3;
}
void DMA2_Stream4_IRQHandler(void)
{
	DMA2_Stream4_handler();

	DMA2->HIFCR=DMA_HIFCR_CFEIF4;
	DMA2->HIFCR=DMA_HIFCR_CDMEIF4;
	DMA2->HIFCR=DMA_HIFCR_CTEIF4;
	DMA2->HIFCR=DMA_HIFCR_CHTIF4;
	DMA2->HIFCR=DMA_HIFCR_CTCIF4;
}

void DMA2_Stream5_IRQHandler(void)
{
	DMA2_Stream5_handler();

	DMA2->HIFCR=DMA_HIFCR_CFEIF5;
	DMA2->HIFCR=DMA_HIFCR_CDMEIF5;
	DMA2->HIFCR=DMA_HIFCR_CTEIF5;
	DMA2->HIFCR=DMA_HIFCR_CHTIF5;
	DMA2->HIFCR=DMA_HIFCR_CTCIF5;
}

void DMA2_Stream6_IRQHandler(void)
{
	DMA2_Stream6_handler();

	DMA2->HIFCR=DMA_HIFCR_CFEIF6;
	DMA2->HIFCR=DMA_HIFCR_CDMEIF6;
	DMA2->HIFCR=DMA_HIFCR_CTEIF6;
	DMA2->HIFCR=DMA_HIFCR_CHTIF6;
	DMA2->HIFCR=DMA_HIFCR_CTCIF6;
}
void DMA2_Stream7_IRQHandler(void)
{
	DMA2_Stream7_handler();

	DMA2->HIFCR=DMA_HIFCR_CFEIF7;
	DMA2->HIFCR=DMA_HIFCR_CDMEIF7;
	DMA2->HIFCR=DMA_HIFCR_CTEIF7;
	DMA2->HIFCR=DMA_HIFCR_CHTIF7;
	DMA2->HIFCR=DMA_HIFCR_CTCIF7;
}
