/*
 * uart_bsp.c
 *
 *  Created on: Jul 11, 2023
 *      Author: hussamaldean
 */



#include "uart_bsp.h"
#include "bsp.h"


static void uart_set_baudrate(USART_TypeDef *USARTx, uint32_t PeriphClk,  uint32_t BaudRate);
static uint16_t compute_uart_bd(uint32_t PeriphClk, uint32_t BaudRate);



void UART2_CLOCK_ENABLE(void)
{
	RCC->APB1ENR|=RCC_APB1ENR_USART2EN;
}

void BSP_UART_Init(USART_TypeDef *uart, UART_ConfigTypedef *config)
{
	switch (config->mode)
	{
		case UART_TX_Only: 	uart->CR1=USART_CR1_TE; break;

		case UART_RX_Only:  uart->CR1=USART_CR1_RE; break;

		case UART_TX_RX:	uart->CR1=USART_CR1_RE|USART_CR1_TE; break;

		default : break;
	}

	uart_set_baudrate(uart, config->busSpeed, config->buadRate);


	if (config->InterrupteEnbable==Interrupt_Enabled)
	{
		switch (config->uart_number)
		{
			case usart2: NVIC_EnableIRQ(USART2_IRQn); break;
			case usart1: NVIC_EnableIRQ(USART1_IRQn); break;
			case uart6: NVIC_EnableIRQ(USART6_IRQn); break;
			default: break;

		}

		if (config->TX_InterruptEnable==TXEIE_Enable)
		{
			uart->CR1|= USART_CR1_TXEIE;
		}

		if(config->RX_InterruptEnable==RXNEIE_Enable)
		{
			uart->CR1|= USART_CR1_RXNEIE;
		}

		if(config->IDLE_Line_Interrupt_Enable==IDLEIE_Enable)
		{
			uart->CR1|=USART_CR1_IDLEIE;
		}
	}


	uart->CR1|= USART_CR1_UE;

}



UART_StatusTypedef BSP_UART_Write_Char(USART_TypeDef *uart,char ch, uint32_t timeout)
{
	uint32_t start_ticks= BSP_Get_Ticks();

	/*Make sure the transmit data register is empty*/
	while(!(uart->SR & USART_SR_TXE))
	{
		if(BSP_Get_Ticks()-start_ticks>timeout)
		{
			return UART_TIMEOUT;
		}
	}

	/*Write to transmit data register*/
	uart->DR	= (ch &0xFF);

	return UART_SUCCESS;
}


UART_StatusTypedef BSP_UART_Read_Char(USART_TypeDef *uart,char *recChar, uint32_t timeout)
{
	uint32_t start_ticks= BSP_Get_Ticks();

	  /*Make sure the receive data register is not empty*/
		while(!(uart->SR & USART_SR_RXNE))
		{
			/*Time out management*/
			if(BSP_Get_Ticks()-start_ticks>timeout)
			{	/*Return if timedout, return timeout*/
				return UART_TIMEOUT;
			}
		}

		/*Read data*/
		*recChar= uart->DR;

		return UART_SUCCESS;

}




static void uart_set_baudrate(USART_TypeDef *USARTx, uint32_t PeriphClk,  uint32_t BaudRate)
{
	USARTx->BRR =  compute_uart_bd(PeriphClk,BaudRate);
}

static uint16_t compute_uart_bd(uint32_t PeriphClk, uint32_t BaudRate)
{
	return ((PeriphClk + (BaudRate/2U))/BaudRate);
}



__WEAK void uart1_interrupt_handler(void){}
__WEAK void uart2_interrupt_handler(void){}
__WEAK void uart6_interrupt_handler(void){}



void USART1_IRQHandler(void)
{
	uart1_interrupt_handler();
	NVIC_ClearPendingIRQ(USART1_IRQn);
}


void USART2_IRQHandler(void)
{
	uart2_interrupt_handler();
	NVIC_ClearPendingIRQ(USART2_IRQn);
}

void USART6_IRQHandler(void)
{
	uart6_interrupt_handler();
	NVIC_ClearPendingIRQ(USART6_IRQn);
}
